from deso.utils import submitTransaction, appendExtraData
import requests
from deso.Sign import Sign_Transaction
from arweave.arweave_lib import Wallet, Transaction
from arweave.transaction_uploader import get_uploader
import arweave
import pathlib


class Social:
    def __init__(
        self,
        publicKey,
        seedHex=None,
        nodeURL="https://node.deso.org/api/v0/",
        derivedPublicKey=None,
        derivedSeedHex=None,
        minFee=1000,
        derivedKeyFee=1700,
    ):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey
        self.NODE_URL = nodeURL
        self.DERIVED_PUBLIC_KEY = derivedPublicKey
        self.DERIVED_SEED_HEX = derivedSeedHex
        self.MIN_FEE = minFee if seedHex else derivedKeyFee

    def submitPost(
        self,
        body,
        imageURLs=[],
        videoURLs=[],
        postHashHexToModify="",
        parentStakeID="",
        isHidden=False,
        repostedPostHash="",
        postExtraData={"App": "DesoPy", "Language": "en"},
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "submit-post"
            finalPostExtraData = postExtraData
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                if "DerivedPublicKey" not in finalPostExtraData:
                    finalPostExtraData[
                        "DerivedPublicKey"
                    ] = self.DERIVED_PUBLIC_KEY

            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "PostHashHexToModify": postHashHexToModify,
                "ParentStakeID": parentStakeID,
                "Title": "",
                "BodyObj": {
                    "Body": body,
                    "ImageURLs": imageURLs,
                    "VideoURLs": videoURLs,
                },
                "RepostedPostHashHex": repostedPostHash,
                "PostExtraData": finalPostExtraData,
                "Sub": "",
                "IsHidden": isHidden,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def repost(
        self,
        postHashHexToRepost,
        postHashHexToModify="",
        parentStakeID="",
        postExtraData={"App": "DesoPy", "Language": "en"},
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "submit-post"
            finalPostExtraData = postExtraData
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                if "DerivedPublicKey" not in finalPostExtraData:
                    finalPostExtraData[
                        "DerivedPublicKey"
                    ] = self.DERIVED_PUBLIC_KEY

            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "PostHashHexToModify": postHashHexToModify,
                "ParentStakeID": parentStakeID,
                "Title": "",
                "BodyObj": {},
                "RepostedPostHashHex": postHashHexToRepost,
                "PostExtraData": finalPostExtraData,
                "Sub": "",
                "IsHidden": False,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
                "InTutorial": False,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def quote(
        self,
        postHashHexToQuote,
        body="",
        imageURLs=[],
        videoURLs=[],
        postExtraData={"App": "DesoPy", "Language": "en"},
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "submit-post"
            finalPostExtraData = postExtraData
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                if "DerivedPublicKey" not in finalPostExtraData:
                    finalPostExtraData[
                        "DerivedPublicKey"
                    ] = self.DERIVED_PUBLIC_KEY

            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "PostHashHexToModify": "",
                "ParentStakeID": "",
                "Title": "",
                "BodyObj": {
                    "Body": body,
                    "ImageURLs": imageURLs,
                    "VideoURLs": videoURLs,
                },
                "RepostedPostHashHex": postHashHexToQuote,
                "PostExtraData": finalPostExtraData,
                "Sub": "",
                "IsHidden": False,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def follow(self, publicKeyToFollow, isFollow=True):
        try:
            error = None
            endpointURL = self.NODE_URL + "create-follow-txn-stateless"
            payload = {
                "FollowerPublicKeyBase58Check": self.PUBLIC_KEY,
                "FollowedPublicKeyBase58Check": publicKeyToFollow,
                "IsUnfollow": not isFollow,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def like(self, postHashHex, isLike=True):
        try:
            error = None
            endpointURL = self.NODE_URL + "create-like-stateless"
            payload = {
                "ReaderPublicKeyBase58Check": self.PUBLIC_KEY,
                "LikedPostHashHex": postHashHex,
                "IsUnlike": not isLike,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def diamond(self, postHashHex, receiverPublicKey, diamondLevel=1):
        try:
            error = None
            endpointURL = self.NODE_URL + "send-diamonds"
            payload = {
                "SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                "DiamondPostHashHex": postHashHex,
                "DiamondLevel": diamondLevel,
                "ReceiverPublicKeyBase58Check": receiverPublicKey,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
                "InTutorial": False,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def updateProfile(
        self,
        FR,
        description,
        username,
        profilePicBase64,
        newStakeMultipleBasisPoint=12500,
        extraData={},
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "update-profile"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
                "NewCreatorBasisPoints": int(FR * 100),
                "NewDescription": description,
                "NewUsername": username,
                "NewProfilePic": profilePicBase64,
                "ExtraData": extraData,
                "NewStakeMultipleBasisPoints": newStakeMultipleBasisPoint,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def sendPrivateMessage(self, receiverPublicKey, message):
        try:
            error = None
            endpointURL = self.NODE_URL + "send-message-stateless"
            payload = {
                "SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                "RecipientPublicKeyBase58Check": receiverPublicKey,
                "MessageText": message,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def mint(
        self,
        postHashHex,
        minBidDeSo,
        copy=1,
        creatorRoyality=5,
        coinHolderRoyality=10,
        isForSale=True,
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "create-nft"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "NFTPostHashHex": postHashHex,
                "NumCopies": copy,
                "NFTRoyaltyToCreatorBasisPoints": round(creatorRoyality * 100),
                "NFTRoyaltyToCoinBasisPoints": round(coinHolderRoyality * 100),
                "HasUnlockable": False,
                "IsForSale": isForSale,
                "MinBidAmountNanos": round(minBidDeSo * 1e9),
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def uploadToArweave(wallet, image):
        wallet = arweave.Wallet(wallet)
        with open(image, "rb", buffering=0) as file_handler:
            tx = Transaction(wallet, file_handler=file_handler, file_path=image)
            file_extension = pathlib.Path(image).suffix
            type = str(file_extension[1:])
            tx.add_tag("Content-Type", "image/" + type)
            tx.sign()
            uploader = get_uploader(tx, file_handler)
            while not uploader.is_complete:
                uploader.upload_chunk()
            tx.send()
            image_id = str(tx.id)
            transaction_id = wallet.get_last_transaction_id()
            build_url = (
                "https://" + transaction_id[1:] + ".arweave.net/" + image_id
            )
            return build_url

    def updateNFT(
        self,
        postHashHex,
        buyNowPriceInDeso,
        buyNow=True,
        minBidDeso=1,
        forSale=True,
        serialNumber=1,
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "update-nft"
            payload = {
                "BuyNowPriceNanos": round(buyNowPriceInDeso * 1e9)
                if buyNow
                else round(minBidDeso * 1e9),
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "IsBuyNow": buyNow,
                "NFTPostHashHex": postHashHex,
                "SerialNumber": serialNumber,
                "IsForSale": forSale,
                "MinBidAmountNanos": None
                if buyNow
                else round(minBidDeso * 1e9),
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def burnNFT(self, postHashHex, serialNumber):
        try:
            error = None
            endpointURL = self.NODE_URL + "burn-nft"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "NFTPostHashHex": postHashHex,
                "SerialNumber": serialNumber,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def createNFTBid(self, bidAmountDeso, NFTPostHashHex, serialNumber):
        try:
            error = None
            endpointURL = self.NODE_URL + "create-nft-bid"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "NFTPostHashHex": NFTPostHashHex,
                "SerialNumber": serialNumber,
                "BidAmountNanos": round(bidAmountDeso * 1e9),
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def transferNFT(self, NFTPostHashHex, receiverPublicKey, serialNumber):
        try:
            error = None
            endpointURL = self.NODE_URL + "transfer-nft"
            payload = {
                "SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                "ReceiverPublicKeyBase58Check": receiverPublicKey,
                "NFTPostHashHex": NFTPostHashHex,
                "SerialNumber": serialNumber,
                "EncryptedUnlockableText": "",
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)

            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])



import requests


class Posts:
    def __init__(self, nodeURL="https://node.deso.org/api/v0/"):
        self.NODE_URL = nodeURL

    def getPostsStateless(
        self,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
        addGlobalFeedBool=False,
        fetchSubcommnets=False,
        getPostsByDESO=False,
        getPostsForFollowFeed=False,
        getPostsForGlobalWhitelist=True,
        mediaRequired=False,
        numToFetch=50,
        orderBy="",
        postContent="",
        postHashHex="",
        postsByDESOMinutesLookBack=0,
        customPayload={},
    ):
        endpointURL = self.NODE_URL + "get-posts-stateless"
        payload = {
            "PostHashHex": postHashHex,
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "OrderBy": orderBy,
            "StartTstampSecs": None,
            "PostContent": postContent,
            "NumToFetch": numToFetch,
            "FetchSubcomments": fetchSubcommnets,
            "GetPostsForFollowFeed": getPostsForFollowFeed,
            "GetPostsForGlobalWhitelist": getPostsForGlobalWhitelist,
            "GetPostsByDESO": getPostsByDESO,
            "MediaRequired": mediaRequired,
            "PostsByDESOMinutesLookback": postsByDESOMinutesLookBack,
            "AddGlobalFeedBool": addGlobalFeedBool,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getSinglePost(
        self,
        postHashHex,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
        fetchParents=False,
        commnetLimit=10,
        commentOffset=0,
        addGlobalFeedBool=False,
        loadAuthorThread=True,
        ThreadLeafLimit=1,
        ThreadLevelLimit=2,
    ):
        endpointURL = self.NODE_URL + "get-single-post"
        payload = {
            "PostHashHex": postHashHex,
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "FetchParents": fetchParents,
            "CommentOffset": commentOffset,
            "CommentLimit": commnetLimit,
            "AddGlobalFeedBool": addGlobalFeedBool,
            "ThreadLevelLimit": ThreadLevelLimit,
            "ThreadLeafLimit": ThreadLeafLimit,
            "LoadAuthorThread": loadAuthorThread,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getPostsForPublicKey(
        self,
        username="",
        publicKey="",
        mediaRequired=False,
        numToFetch=10,
        lastPostHashHex="",
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        endpoint = self.NODE_URL + "get-posts-for-public-key"
        payload = {
            "PublicKeyBase58Check": publicKey,
            "Username": username,
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "LastPostHashHex": lastPostHashHex,
            "NumToFetch": numToFetch,
            "MediaRequired": mediaRequired,
        }
        response = requests.post(endpoint, json=payload)
        return response

    def getDiamondsForPost(
        self,
        postHashHex,
        limit=25,
        offset=0,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        endpointURL = self.NODE_URL + "get-diamonds-for-post"
        payload = {
            "PostHashHex": postHashHex,
            "Offset": offset,
            "Limit": limit,
            "ReaderPublicKeyBase58Check": readerPublicKey,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getLikesForPost(
        self,
        postHashHex,
        limit=50,
        offset=0,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        endpointURL = self.NODE_URL + "get-likes-for-post"
        payload = {
            "PostHashHex": postHashHex,
            "Offset": offset,
            "Limit": limit,
            "ReaderPublicKeyBase58Check": readerPublicKey,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getQuoteRepostsForPost(
        self,
        postHashHex,
        limit=50,
        offset=0,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        endpointURL = self.NODE_URL + "get-quote-reposts-for-post"
        payload = {
            "PostHashHex": postHashHex,
            "Offset": offset,
            "Limit": limit,
            "ReaderPublicKeyBase58Check": readerPublicKey,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getNFTEntriesForNFTPost(
        self,
        postHashHex,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        endpointURL = self.NODE_URL + "get-nft-entries-for-nft-post"
        payload = {
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "PostHashHex": postHashHex,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getNFTBidsForNFTPost(
        self,
        postHashHex,
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        endpointURL = self.NODE_URL + "get-nft-bids-for-nft-post"
        payload = {
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "PostHashHex": postHashHex,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getHotFeed(
        self,
        taggedUsername,
        responeLimit=10,
        sortByNew=True,
        seenPosts=[],
        readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop",
    ):
        """Returns posts that has mentioned in username"""
        endpointURL = self.NODE_URL + "get-hot-feed"
        payload = {
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "SeenPosts": seenPosts,
            "Tag": f"@{taggedUsername}",
            "SortByNew": sortByNew,
            "ResponseLimit": responeLimit,
        }

        response = requests.post(endpointURL, json=payload)
        return response



from deso.utils import getUserJWT
import requests


class Metadata:
    def __init__(
        self,
        publicKey="BC1YLhBLE1834FBJbQ9JU23JbPanNYMkUsdpJZrFVqNGsCe7YadYiUg",
        nodeURL="https://node.deso.org/api/v0/",
    ):
        self.PUBLIC_KEY = publicKey
        self.NODE_URL = nodeURL

    def getNodeHealth(self):
        endpointURL = self.NODE_URL + "health-check"
        response = requests.get(endpointURL)
        return response

    def getExchangeRate(self):
        """Returns Deso Price"""
        endpointURL = self.NODE_URL + "get-exchange-rate"
        response = requests.get(endpointURL)
        return response

    def getAppState(self):
        """Returns App State that includes current block height, deso price and other node related data"""
        payload = {"PublicKeyBase58Check": self.PUBLIC_KEY}

        endpointURL = self.NODE_URL + "get-app-state"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDiamondLevelMap(self, inDesoNanos=True):
        """Returns Diamond Level Map. set inDesoNanos = False to get in Deso"""
        if inDesoNanos:
            return {
                "1": 50000,
                "2": 500000,
                "3": 5000000,
                "4": 50000000,
                "5": 500000000,
                "6": 5000000000,
                "7": 50000000000,
                "8": 500000000000,
            }

        else:
            return {
                "1": 0.00005,
                "2": 0.0005,
                "3": 0.005,
                "4": 0.05,
                "5": 0.5,
                "6": 5,
                "7": 50,
                "8": 500,
            }



from deso.utils import submitTransaction, appendExtraData
import requests
from deso.Sign import Sign_Transaction


class Trade:
    def __init__(
        self,
        publicKey,
        seedHex=None,
        nodeURL="https://node.deso.org/api/v0/",
        derivedPublicKey=None,
        derivedSeedHex=None,
        minFee=1000,
        derivedKeyFee=1700,
    ):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey
        self.NODE_URL = nodeURL
        self.DERIVED_PUBLIC_KEY = derivedPublicKey
        self.DERIVED_SEED_HEX = derivedSeedHex
        self.MIN_FEE = minFee if seedHex else derivedKeyFee

    def sendDeso(self, recieverPublicKeyOrUsername, desoToSend):
        try:
            error = None
            endpointURL = self.NODE_URL + "send-deso"
            payload = {
                "SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                "RecipientPublicKeyOrUsername": recieverPublicKeyOrUsername,
                "AmountNanos": int(round(desoToSend * 1e9)),
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def buyCreatorCoin(self, creatorPublicKey, desoAmountToBuy):
        try:
            error = None
            endpointURL = self.NODE_URL + "buy-or-sell-creator-coin"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "CreatorPublicKeyBase58Check": creatorPublicKey,
                "OperationType": "buy",
                "DeSoToSellNanos": int(desoAmountToBuy * 1e9),
                "CreatorCoinToSellNanos": 0,
                "DeSoToAddNanos": 0,
                "MinDeSoExpectedNanos": 0,
                "MinCreatorCoinExpectedNanos": 0,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
                "InTutorial": False,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def getHeldCoinsOfCreator(self, publicKeyOfCoin):
        endpoint = self.NODE_URL + "get-users-stateless"
        payload = {"PublicKeysBase58Check": [self.PUBLIC_KEY]}
        response = requests.post(endpoint, json=payload)
        hodlings = response.json()["UserList"][0]["UsersYouHODL"]
        for hodling in hodlings:
            if hodling["CreatorPublicKeyBase58Check"] == publicKeyOfCoin:
                coinsHeld = hodling["BalanceNanos"]
                if coinsHeld != 0:
                    return hodling["BalanceNanos"]
                else:
                    return -1
        return -1

    def amountOnSell(desoLockedNanos, coinsInCirculation, balanceNanos):
        beforeFees = desoLockedNanos * (
            1 - pow((1 - balanceNanos / coinsInCirculation), (1 / 0.3333333))
        )
        return (beforeFees * (100 * 100 - 1)) / (100 * 100)

    def sellCreatorCoin(self, creatorPublicKey, coinsToSellNanos):
        try:
            endpointURL = self.NODE_URL + "buy-or-sell-creator-coin"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "CreatorPublicKeyBase58Check": creatorPublicKey,
                "OperationType": "sell",
                "DeSoToSellNanos": 0,
                "CreatorCoinToSellNanos": coinsToSellNanos,
                "DeSoToAddNanos": 0,
                "MinDeSoExpectedNanos": 0,
                "MinCreatorCoinExpectedNanos": 0,
                "MinFeeRateNanosPerKB": 1000,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def sendCreatorCoins(
        self,
        creatorPublicKey,
        receiverUsernameOrPublicKey,
        creatorCoinNanosToSend,
    ):
        try:
            error = None
            endpointURL = self.NODE_URL + "transfer-creator-coin"
            payload = {
                "SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                "CreatorPublicKeyBase58Check": creatorPublicKey,
                "ReceiverUsernameOrPublicKeyBase58Check": receiverUsernameOrPublicKey,
                "CreatorCoinToTransferNanos": creatorCoinNanosToSend,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def sendDAOCoins(
        self, coinsToTransfer, daoPublicKeyOrName, receiverPublicKeyOrUsername
    ):
        """Sends DAO coin to publicKey or username. Use the hex() function to convert a number to hexadecimal
        for Example, if you want to send 15 DAO coin, set coinsToTransfer to hex(int(15*1e18))"""
        try:
            error = None
            endpointURL = self.NODE_URL + "transfer-dao-coin"
            payload = {
                "SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                "ProfilePublicKeyBase58CheckOrUsername": daoPublicKeyOrName,
                "ReceiverPublicKeyBase58CheckOrUsername": receiverPublicKeyOrUsername,
                "DAOCoinToTransferNanos": str(coinsToTransfer),
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }

            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def burnDAOCoins(self, coinsToBurn, daoPublicKeyOrName):
        """Burns DAO coin of daoPublicKeyOrName. Use the hex() function to convert a number to hexadecimal
        for Example, if you want to burn 15 DAO coin, set coinsToBurn to hex(int(15*1e18))"""
        try:
            error = None
            endpointURL = self.NODE_URL + "dao-coin"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "ProfilePublicKeyBase58CheckOrUsername": daoPublicKeyOrName,
                "OperationType": "burn",
                "CoinsToBurnNanos": coinsToBurn,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }

            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def mintDAOCoins(self, coinsToMint):
        """Mint DAO coins. Use the hex() function to convert a number to hexadecimal
        for Example, if you want to mint 15 DAO coin, set coinsToBurn to hex(int(15*1e18))"""
        try:
            error = None
            endpointURL = self.NODE_URL + "dao-coin"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "ProfilePublicKeyBase58CheckOrUsername": self.PUBLIC_KEY,
                "OperationType": "mint",
                "CoinsToMintNanos": coinsToMint,
                "MinFeeRateNanosPerKB": self.MIN_FEE,
            }

            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if (
                self.DERIVED_PUBLIC_KEY is not None
                and self.DERIVED_SEED_HEX is not None
                and self.SEED_HEX is None
            ):
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL
                )
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = (
                self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            )
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex
                )
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."
                }
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL
            )
            return submitTransactionResponse

        except Exception as e:
            raise Exception(error["error"])



import requests


class User:
    def __init__(self, nodeURL="https://node.deso.org/api/v0/"):
        self.NODE_URL = nodeURL

    def getSingleProfile(
        self, username="", publicKey="", NoErrorOnMissing=False
    ):
        """Gives user profile info."""
        endpointURL = self.NODE_URL + "get-single-profile"
        payload = {
            "PublicKeyBase58Check": publicKey,
            "Username": username,
            "NoErrorOnMissing": NoErrorOnMissing,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getUsersStateless(self, listOfPublicKeys, skipForLeaderboard=True):
        """Returns information about list of PublicKeys"""
        endpointURL = self.NODE_URL + "get-users-stateless"
        payload = {
            "PublicKeysBase58Check": listOfPublicKeys,
            "SkipForLeaderboard": skipForLeaderboard,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getProfilePicURL(self, publicKey):
        """Returns the profile pic URL for a public key"""
        profilePicURL = f"{self.NODE_URL}get-single-profile-picture/{publicKey}?fallback=https://node.deso.org/assets/img/default_profile_pic.png"
        return profilePicURL

    def getMessagesStateless(
        self,
        publicKey,
        numToFetch=25,
        sortAlgorithm="time",
        followersOnly=False,
        followingOnly=False,
        holdersOnly=False,
        holdingsOnly=False,
        fetchAfterPublicKey="",
    ):
        endpointURL = self.NODE_URL + "get-messages-stateless"
        payload = {
            "PublicKeyBase58Check": publicKey,
            "FetchAfterPublicKeyBase58Check": fetchAfterPublicKey,
            "NumToFetch": numToFetch,
            "HoldersOnly": holdersOnly,
            "HoldingsOnly": holdingsOnly,
            "FollowersOnly": followersOnly,
            "FollowingOnly": followingOnly,
            "SortAlgorithm": sortAlgorithm,
        }
        response = requests.post(endpointURL, json=payload)
        return response

    def getNotifications(
        self,
        publicKey,
        startIndex=-1,
        numToFetch=50,
        filterOutNotificationCategories={},
    ):
        # filterOutNotification is a map that looks like {"diamond": True, "like": True,  "transfer": True, "follow": True, "nft": True, "post": True}
        # ever True means that the specific notification category will be filtered out
        payload = {
            "PublicKeyBase58Check": publicKey,
            "FetchStartIndex": startIndex,
            "NumToFetch": numToFetch,
            "FilteredOutNotificationCategories": filterOutNotificationCategories,
        }
        endpointURL = "https://diamondapp.com/api/v0/get-notifications"
        response = requests.post(endpointURL, json=payload)
        return response

    def getNFTs(self, userPublicKey, readerPublicKey="", isForSale=False):
        """Gets the NFTs associated with a user,
        setting isForSale = True returns only the NFTs that are for sale."""
        payload = {
            "UserPublicKeyBase58Check": userPublicKey,
            "ReaderPublicKeyBase58Check": readerPublicKey,
            "IsForSale": isForSale,
        }

        endpointURL = self.NODE_URL + "get-nfts-for-user"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDerivedKeys(self, publicKey):
        payload = {"PublicKeyBase58Check": publicKey}
        endpointURL = self.NODE_URL + "get-user-derived-keys"
        response = requests.post(endpointURL, json=payload)
        return response

    def getTransactionInfo(
        self,
        publicKey,
        limit=200,
        lastTransactionIDBase58Check="",
        lastPublicKeyTransactionIndex=-1,
    ):
        payload = {
            "PublicKeyBase58Check": publicKey,
            "LastTransactionIDBase58Check": lastTransactionIDBase58Check,
            "LastPublicKeyTransactionIndex": lastPublicKeyTransactionIndex,
            "Limit": limit,
        }
        endpointURL = self.NODE_URL + "transaction-info"
        response = requests.post(endpointURL, json=payload)
        return response

    def getHoldersForPublicKey(
        self,
        publicKey="",
        username="",
        fetchAll=False,
        numToFetch=100,
        fetchHodlings=False,
        isDAOCOIN=False,
        lastPublicKey="",
    ):
        payload = {
            "PublicKeyBase58Check": publicKey,
            "Username": username,
            "LastPublicKeyBase58Check": lastPublicKey,
            "NumToFetch": numToFetch,
            "FetchHodlings": fetchHodlings,
            "FetchAll": fetchAll,
            "IsDAOCoin": isDAOCOIN,
        }

        endpointURL = self.NODE_URL + "get-hodlers-for-public-key"
        response = requests.post(endpointURL, json=payload)
        return response

    def getFollowsStateless(
        self,
        username="",
        publicKey="",
        getFollowing=True,
        numToFetch=50,
        lastPublicKey="",
    ):
        """Returns list of user followings and followers"""
        payload = {
            "Username": username,
            "PublicKeyBase58Check": publicKey,
            "GetEntriesFollowingUsername": not getFollowing,
            "LastPublicKeyBase58Check": lastPublicKey,
            "NumToFetch": numToFetch,
        }

        endpointURL = self.NODE_URL + "get-follows-stateless"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDaoCoinLimitOrders(self, daoCoinPublicKey):
        payload = {
            "DAOCoin1CreatorPublicKeyBase58Check": daoCoinPublicKey,
            "DAOCoin2CreatorPublicKeyBase58Check": "DESO",
        }
        endpointURL = self.NODE_URL + "get-dao-coin-limit-orders"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDaoCoinPrice(self, daoCoinPublicKey, type="MARKET"):
        # returns dao coin price in DESO
        if type == "MARKET":
            daoLimtOrders = self.getDaoCoinLimitOrders(
                daoCoinPublicKey=daoCoinPublicKey
            ).json()
            lowestAsk = 0
            highestBid = 0
            orderList = daoLimtOrders["Orders"]
            for orders in orderList:
                operationType = orders["OperationType"]
                ExchangeRateCoinsToSellPerCoinToBuy = orders[
                    "ExchangeRateCoinsToSellPerCoinToBuy"
                ]
                daodaoPriceInDeso = 1 / ExchangeRateCoinsToSellPerCoinToBuy
                if operationType == "ASK":
                    if daodaoPriceInDeso < lowestAsk or lowestAsk == 0:
                        lowestAsk = daodaoPriceInDeso
                if operationType == "BID":
                    if (
                        ExchangeRateCoinsToSellPerCoinToBuy > highestBid
                        or highestBid == 0
                    ):
                        highestBid = ExchangeRateCoinsToSellPerCoinToBuy

            return (lowestAsk + highestBid) / 2

    def getDiamondsForPublicKey(self, publicKey, received=True):
        """Returns diamonds received/given by publicKey."""
        payload = {
            "PublicKeyBase58Check": publicKey,
            "FetchYouDiamonded": not received,
        }
        endpointURL = self.NODE_URL + "get-diamonds-for-public-key"
        response = requests.post(endpointURL, json=payload)
        return response



from deso.utils import getUserJWT
import requests


class Media:
    def __init__(
        self,
        publicKey=None,
        seedHex=None,
        nodeURL="https://node.deso.org/api/v0/",
    ):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey
        self.NODE_URL = nodeURL

    def uploadImage(self, fileList):
        # uploads image to images.deso.org
        try:
            if type(fileList) == type("str"):
                fileList = [
                    ("file", (fileList, open(fileList, "rb"), "image/png"))
                ]

            jwt_token = getUserJWT(self.SEED_HEX)
            # print(encoded_jwt)
            endpointURL = self.NODE_URL + "upload-image"
            payload = {
                "UserPublicKeyBase58Check": self.PUBLIC_KEY,
                "JWT": jwt_token,
            }

            response = requests.post(endpointURL, data=payload, files=fileList)
            return response
        except Exception as e:
            return e



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = "2.1.10"
DESCRIPTION = "A python module for DeSo"
LONG_DESCRIPTION = "DesoPy is a python module that enables devs to interact with DeSo Blockchain using node.deso.org node by default."
# Setting up
setup(
    name="deso",
    version=VERSION,
    author="ItsAditya (https://diamondapp.com/u/ItsAditya)",
    author_email="<chaudharyaditya0005@gmail.com>",
    description="A python module for deso",
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=["PyJWT", "ecdsa", "arweave-python-client", "base58"],
    keywords=[
        "deso",
        "python",
        "bitclout",
        "social media",
        "crypto",
        "blockchain",
        "decentralisation",
    ],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
)
